import os
import sys
from recycle.lib import my_print, HOME, my_input

shell_config = {
    "zsh": "compdef '_files -W ~/.Trash`pwd`' undel\n",
    "bash": """_undel() {
    local trash_home="$HOME/.Trash"
    local current_path=`pwd`
    local current_arg="${COMP_WORDS[COMP_CWORD]}"
    local trash_dir=$trash_home$current_path/${COMP_LINE:6:999}
    local IFS=$'\t\n'

    if [[ $current_arg =~ ^/ ]] ; then
      # /a/b/c
      return 0
    elif [[ $current_path =~ $trash_home ]] ; then
      # in /root/.Trash/
      return 0
    elif [[ ! -d $trash_dir ]] ; then
      # /root/.Trash/`pwd` don't exists
      return 0
    elif [[ -d $trash_dir ]] ; then
      COMPREPLY=( $(compgen -W "`ls -a $trash_dir|grep -Pv '^\.{1,2}$'`" -- ${current_arg}) )
      return 0
    fi
}

complete -o filenames -o dirnames -o default -F _undel undel
""",
}


def install(shell, path):
    shellrc_path = "{}/.{}rc".format(HOME, shell)

    if not os.path.exists("/bin/" + shell):
        return my_print("your don't have {}, skip it".format(shell))

    open(shellrc_path, 'a+').close()

    if "undel" not in open(shellrc_path, "r").read():
        open(shellrc_path, "a+").write(shell_config[shell])
        my_print("Installed in {}, enjoy it :)".format(shellrc_path))

    if not path:
        return

    path = "export PATH={}\n".format(path)
    if path in open(shellrc_path).readlines():
        return
    my_print('Write "{}" in {}'.format(path.strip(), shellrc_path))
    open(shellrc_path, "a+").write(path)

def main():
    scripts_dir_path = os.path.dirname(sys.argv[0])
    if scripts_dir_path in os.getenv("PATH"):
        path = None
    elif (
        "n"
        in my_input(
            'Add "{}" to your PATH, to enable it?[Y/n]'.format(scripts_dir_path)
        ).lower()
    ):
        path = None
    else:
        path = "{}:$PATH".format(scripts_dir_path)

    install("zsh", path)
    install("bash", path)
    command = "source ~/.{}rc".format(os.getenv("SHELL").split("/")[-1])
    my_print('\nPlease Execute "{}"\n'.format(command))



# coding: utf-8

import os
import re

from recycle.lib import (
    TRASH_PATH,
    TRASH_REGEX,
    operations,
    directory_exists,
    search_files,
    replace_file,
    execute_move,
    remove_empty_dir,
)


def recover_trash_file(trash_dir, file_regex, reverse):
    recover_dir = "/" + trash_dir[len(TRASH_PATH) :].strip("/")
    trash_file_name = file_regex[1:-1]
    absolute_file = os.path.join(trash_dir, trash_file_name)

    if re.search(TRASH_REGEX, trash_file_name) and replace_file(recover_dir):
        execute_move(absolute_file, recover_dir)
        return remove_empty_dir(trash_dir)

    for file_name in search_files(trash_dir, file_regex):
        recover_path = os.path.join(recover_dir, trash_file_name)
        if replace_file(recover_path):
            for file_id in search_files(absolute_file, TRASH_REGEX):
                return recover_by_id(file_id, recover_path, absolute_file)
        return


def recover_by_id(file_id, recover_path, absolute_dir):
    if replace_file(recover_path):
        absolute_file = os.path.join(absolute_dir, file_id)
        execute_move(absolute_file, recover_path)
        return remove_empty_dir(absolute_dir)


def recover_by_regrex(absolute_dir, file_regex, recover_path, reverse):
    for file_name_dir in search_files(absolute_dir, file_regex):
        recover_trash_file_dir = os.path.join(recover_path, file_name_dir)
        absolute_trash_file_dir = os.path.join(absolute_dir, file_name_dir)
        if not directory_exists(absolute_trash_file_dir):
            return
        trash_files_list = search_files(absolute_trash_file_dir, TRASH_REGEX)
        for file_name in sorted(trash_files_list, reverse=reverse):
            recover_by_id(file_name, recover_trash_file_dir, absolute_trash_file_dir)
            break
        remove_empty_dir(absolute_trash_file_dir)


def recover_from_trash(trash_dir, file_regex, reverse):
    relative_dir = trash_dir.strip("/")
    absolute_dir = os.path.join(TRASH_PATH, relative_dir)
    recover_path = "/" + relative_dir

    if trash_dir.startswith(TRASH_PATH):
        # Current dir is TRASH_PATH
        return recover_trash_file(trash_dir, file_regex, reverse)

    if not directory_exists(absolute_dir):
        return

    if re.search(TRASH_REGEX, file_regex[1:-1]):
        return recover_by_id(file_regex[1:-1], recover_path, absolute_dir)

    recover_by_regrex(absolute_dir, file_regex, recover_path, reverse)


def main():
    for parent_dir, file_regex, reverse in operations():
        recover_from_trash(parent_dir, file_regex, reverse)



# coding: utf-8

import os
import sys
import re
import datetime
import subprocess

HOME = os.path.expanduser("~")
TRASH_PATH = HOME + "/.Trash"
TRASH_DATETIME_FORMAT = "%Y-%m-%d_%H:%M:%S%f_"
TRASH_REGEX = "^\d{4}-\d{2}-\d{2}_\d{2}:\d{2}:\d{8}_[0-9\.]+[A-Z]?$"


def get_current_path():
    try:
        return os.path.abspath(os.getcwd())
    except Exception:
        return None


def my_print(s):
    sys.stdout.write("{}\n".format(s))


def get_path_size_str(file_path):
    return subprocess.check_output(["du", "-sh", file_path]).split()[0].decode("utf-8")


def generate_trash_file_name(file_path):
    return datetime.datetime.now().strftime(TRASH_DATETIME_FORMAT) + get_path_size_str(
        file_path
    )


def directory_exists(directory):
    if not os.path.isdir(directory):
        my_print("{} not exists.".format(directory))
        return False
    return True


def search_files(directory, file_regex):
    files = []
    file_regex = (
        file_regex.replace("(", r"\(")
        .replace(")", r"\)")
        .replace(r"\\(", r"\(")
        .replace(r"\\)", r"\)")
    )
    if not directory_exists(directory):
        return files

    for file_name in os.listdir(directory):
        if re.search(file_regex, file_name):
            files.append(file_name)
    return files


def execute_move(source, destination):
    my_print("mv {} {}".format(source, destination))
    source = re.escape(source)
    destination = re.escape(destination)
    os.system("mv {} {}".format(source, destination))


def remove_empty_dir(absolute_path):
    while os.path.isdir(absolute_path) and not os.listdir(absolute_path):
        os.rmdir(absolute_path)
        absolute_path = absolute_path[: absolute_path.rindex("/")]
        if absolute_path == TRASH_PATH:
            break

def my_input(s):
    try:
        return raw_input(s)
    except:
        return input(s)


def replace_file(file_path):
    if os.path.exists(file_path):
        if (
            "n"
            not in my_input(
                "{} already exists replace it? [Y/n]".format(file_path)
            ).lower()
        ):
            trash_file = os.path.join(
                TRASH_PATH + file_path, generate_trash_file_name(file_path)
            )
            execute_move(file_path, trash_file)
            return True
    else:
        return True


def get_absolute_dirs(dirs):
    absolute_dirs = []

    for i in dirs:
        if i == "..":
            absolute_dirs.pop(-1)
        elif absolute_dirs and not (i or absolute_dirs[-1]):
            pass
        else:
            absolute_dirs.append(i)
    return absolute_dirs


def get_parent_dir_and_file_regex(input_arg):

    parent_dir = get_current_path()

    dirs = input_arg.split("/")
    reverse = True

    if dirs:
        if not dirs[0]:
            # use absolute path
            dirs = get_absolute_dirs(dirs)
        else:
            dirs = get_absolute_dirs(parent_dir.split("/") + dirs)
        parent_dir = "/".join(dirs[:-1]) or "/"

    file_regex = dirs[-1] if dirs else input_arg
    return parent_dir, file_regex, reverse


def operations():
    if not os.path.exists(TRASH_PATH):
        os.mkdir(TRASH_PATH)

    for arg in sys.argv[1:]:
        arg = arg.strip().rstrip("/")
        arg = arg[2:] if arg[:2] == "./" else arg
        parent_dir, file_regex, reverse = get_parent_dir_and_file_regex(arg)
        if not arg:
            continue

        if "^" != file_regex[0] and "$" != file_regex[-1]:
            file_regex = "^{}$".format(file_regex)
        yield parent_dir, file_regex, reverse
